#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

# disable: accessing protected members, too many methods
# pylint: disable=W0212,R0904
import contextlib
import unittest

from hamcrest import is_
from hamcrest import contains_exactly as contains
from hamcrest import assert_that

import fudge


from zope import component
from zope import interface

from zc.displayname.interfaces import IDisplayNameGenerator

from nti.mailer._compat import parseaddr

from nti.mailer._verp import verp_from_recipients
from nti.mailer._verp import principal_ids_from_verp

from nti.mailer.interfaces import EmailAddressablePrincipal


def _make_displayname_adapter(display_name):

    @interface.implementer(IDisplayNameGenerator)
    class StaticDisplayNameAdapter(object):

        def __init__(self, _site, _request):
            pass

        def __call__(self):
            return display_name

    return StaticDisplayNameAdapter


class TestVerp(unittest.TestCase):

    def test_pids_from_verp_email(self):
        fromaddr = 'no-reply+kaley.white%40nextthought.com.PFgX7A@nextthought.com'
        pids = principal_ids_from_verp(fromaddr, default_key='alpha.nextthought.com')
        assert_that(pids, contains('kaley.white@nextthought.com'))

        # With label
        fromaddr = 'no-reply+label+label2+kaley.white%40nextthought.com.PFgX7A@nextthought.com'
        pids = principal_ids_from_verp(fromaddr, default_key='alpha.nextthought.com')
        assert_that(pids, contains('kaley.white@nextthought.com'))

        # With '+' in principal
        fromaddr = 'no-reply+label+foo%2B%2B%2B.PXAYJg@nextthought.com'
        pids = principal_ids_from_verp(fromaddr, default_key='alpha.nextthought.com')
        assert_that(pids, contains('foo+++'))

        pids = principal_ids_from_verp(fromaddr)
        assert_that(pids, is_(()))

    @fudge.patch('nti.mailer._verp._get_default_sender',
                             'nti.mailer._verp._get_signer_secret')
    def test_verp_from_recipients_in_site_uses_default_sender_realname(self, mock_find, mock_secret):
        mock_find.is_callable().returns('Janux <janux@ou.edu>')
        mock_secret.is_callable().returns('abc123')

        prin = self.email_addr_principal('foo', 'foo@bar.com')

        addr = verp_from_recipients('no-reply@nextthought.com',
                                    (prin,),
                                    default_key='alpha.nextthought.com')

        name, email = parseaddr(addr)

        assert_that(name, is_('Janux'))
        assert_that(email, is_('no-reply+foo.UGQXuA@nextthought.com'))

        pids = principal_ids_from_verp(addr, default_key='alpha.nextthought.com')
        assert_that(pids, contains(prin.id))

        # Test with label already; principal with '+' chars.
        prin.id = 'foo+++'
        addr = verp_from_recipients('no-reply+label+label2@nextthought.com',
                                    (prin,),
                                    default_key='alpha.nextthought.com')

        name, email = parseaddr(addr)

        assert_that(name, is_('Janux'))
        assert_that(email, is_('no-reply+label+label2+foo%2B%2B%2B.ULoYGw@nextthought.com'))

        pids = principal_ids_from_verp(addr, default_key='alpha.nextthought.com')
        assert_that(pids, contains(prin.id))

        # If more than one prin, no verp
        prin2 = self.email_addr_principal('foo2', 'foo2@bar.com')

        addr = verp_from_recipients('no-reply+label+label2@nextthought.com',
                                    (prin, prin2),
                                    default_key='alpha.nextthought.com')

        name, email = parseaddr(addr)

        assert_that(name, is_('Janux'))
        assert_that(email, is_('no-reply+label+label2@nextthought.com'))

    def email_addr_principal(self, principal_id, email_address):
        prin = EmailAddressablePrincipal.__new__(EmailAddressablePrincipal)
        prin.email = '%s' % email_address
        prin.id = principal_id
        return prin

    @fudge.patch('nti.mailer._verp._get_default_sender',
                 'nti.mailer._verp._get_signer_secret',
                 'nti.mailer._verp.get_current_request')
    def test_verp_from_recipients_no_default_sender_realname(self,
                                                             mock_find,
                                                             mock_secret,
                                                             get_current_request):
        """
        If there's no realname for the default sender, use the
        brand name as a fallback.
        """
        mock_find.is_callable().returns('janux@ou.edu')
        mock_secret.is_callable().returns('abc123')
        request = fudge.Fake('Request')

        # We'll pass it in the first time through, so this shouldn't
        # be called
        get_current_request.is_callable().returns(request).times_called(0)

        # If we have a IDisplayNameGenerator registered for the current site
        # we'll use that for the real name
        displayname_adapter = _make_displayname_adapter("Brand XYZ")
        with provide_adapter(displayname_adapter,
                             required=(object, object),
                             provided=IDisplayNameGenerator):

            # Explicitly provide request
            prin = self.email_addr_principal('foo', 'foo@bar.com')
            addr = verp_from_recipients('no-reply@nextthought.com',
                                        (prin,),
                                        default_key='alpha.nextthought.com',
                                        request=request)

            name, email = parseaddr(addr)

            assert_that(name, is_('Brand XYZ'))
            assert_that(email, is_('no-reply+foo.UGQXuA@nextthought.com'))

            # Use current request implicitly
            get_current_request.times_called(1)
            addr = verp_from_recipients('no-reply@nextthought.com',
                                        (prin,),
                                        default_key='alpha.nextthought.com')

            name, email = parseaddr(addr)

            assert_that(name, is_('Brand XYZ'))
            assert_that(email, is_('no-reply+foo.UGQXuA@nextthought.com'))

        # But if we don't have an IDisplayNameGenerator, we default to NextThought
        prin = self.email_addr_principal('foo', 'foo@bar.com')
        addr = verp_from_recipients('no-reply@nextthought.com',
                                    (prin,),
                                    default_key='alpha.nextthought.com',
                                    request=request)

        name, email = parseaddr(addr)

        assert_that(name, is_('NextThought'))
        assert_that(email, is_('no-reply+foo.UGQXuA@nextthought.com'))


@contextlib.contextmanager
def provide_adapter(adapter, **kwargs):
    gsm = component.getGlobalSiteManager()
    gsm.registerAdapter(adapter, **kwargs)
    try:
        yield
    finally:
        gsm.unregisterAdapter(adapter, **kwargs)



# -*- coding: utf-8 -*-
#
# nti.identifiers documentation build configuration file, created by
# sphinx-quickstart on Thu Jun  8 07:02:25 2017.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# import os
# import sys
# sys.path.insert(0, os.path.abspath('.'))
import os
import sys
import pkg_resources
sys.path.append(os.path.abspath('../src'))
rqmt = pkg_resources.require('nti.mailer')[0]

# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'
# 1.8 was the last version that runs on Python 2; 2.0+ requires Python 3.
# `autodoc_default_options` was new in 1.8
needs_sphinx = "1.8"

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.doctest',
    'sphinx.ext.coverage',
    'sphinx.ext.intersphinx',
    'sphinx.ext.extlinks',
    'sphinx.ext.viewcode',

    # Third-party
    'repoze.sphinx.autointerface',
    'sphinxcontrib.programoutput',

    # Ours

]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = u'nti.mailer'
copyright = u'2017, NextThought'
author = u'NextThought'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = '%s.%s' % tuple(map(int, rqmt.version.split('.')[:2]))
# The full version, including alpha/beta/rc tags.
release = rqmt.version

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = None

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = False


# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'sphinx_rtd_theme'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#
# html_theme_options = {}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']


# -- Options for HTMLHelp output ------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = 'ntimailerdoc'


# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',

    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',

    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (master_doc, 'ntimailer.tex', u'nti.mailer Documentation',
     u'NextThought', 'manual'),
]


# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (master_doc, 'ntimailer', u'nti.identifiers Documentation',
     [author], 1)
]


# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (master_doc, 'ntimailer', u'nti.mailer Documentation',
     author, 'ntimailer', 'One line description of project.',
     'Miscellaneous'),
]




# Example configuration for intersphinx: refer to the Python standard library.
intersphinx_mapping = {
    "http://docs.pylonsproject.org/projects/pyramid/en/latest/": None,
    #'https://ntiwref.readthedocs.io/en/latest': None,
    'http://docs.python.org/': None,
    'http://ntischema.readthedocs.io/en/latest/': None,
    'http://ntizodb.readthedocs.io/en/latest/': None,
    'http://persistent.readthedocs.io/en/latest': None,
    'http://zodb.readthedocs.io/en/latest': None,

    'http://zopecomponent.readthedocs.io/en/latest': None,
    'http://zopecontainer.readthedocs.io/en/latest': None,
    'http://zopedatetime.readthedocs.io/en/latest': None,
    'http://zopedublincore.readthedocs.io/en/latest': None,
    'http://zopeevent.readthedocs.io/en/latest': None,
    'http://zopehookable.readthedocs.io/en/latest': None,
    'http://zopeinterface.readthedocs.io/en/latest': None,
    'http://zopeintid.readthedocs.io/en/latest/': None,
    'http://zopemimetype.readthedocs.io/en/latest/': None,
    'http://zopeproxy.readthedocs.io/en/latest': None,
    'http://zopeschema.readthedocs.io/en/latest/': None,
    'https://zopesecurity.readthedocs.io/en/latest/': None,
    'http://zopelifecycleevent.readthedocs.io/en/latest/': None,

    'https://docs.pylonsproject.org/projects/pyramid_mailer/en/latest/': None,
    'https://repozesendmail.readthedocs.io/en/latest/': None,
    'https://boto3.amazonaws.com/v1/documentation/api/latest/': None,
}


extlinks = {
    'issue': ('https://github.com/NextThought/nti.mailer/issues/%s',
              'issue #'),
    'pr': ('https://github.com/NextThought/nti.mailer/pull/%s',
           'pull request #')}


# Sphinx 1.8+ prefers this to `autodoc_default_flags`. It's documented that
# either True or None mean the same thing as just setting the flag, but
# only None works in 1.8 (True works in 2.0)
autodoc_default_options = {
    'members': None,
    'show-inheritance': None,
}

autoclass_content = 'both'
autodoc_member_order = 'bysource'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Implementation of the :class:`nti.mailer.interfaces.IVERP` protocol.

.. $Id$
"""

from __future__ import print_function, absolute_import, division

from itsdangerous.exc import BadSignature

from itsdangerous.signer import Signer
from pyramid.threadlocal import get_current_request
from zc.displayname.interfaces import IDisplayNameGenerator

from zope import component
from zope import interface
from zope.component.hooks import getSite

from zope.security.interfaces import IPrincipal

from nti.mailer._compat import parseaddr
from nti.mailer._compat import formataddr

from nti.mailer.interfaces import IVERP
from nti.mailer.interfaces import IMailerPolicy
from nti.mailer.interfaces import IEmailAddressable

logger = __import__('logging').getLogger(__name__)


def _get_signer_secret(default_secret="$Id$"):
    policy = component.queryUtility(IMailerPolicy)
    result = None
    if policy is not None:
        result = policy.get_signer_secret()
    return result or default_secret


import zlib
import struct

def _to_bytes(s, encoding='ascii'):
    # The default encoding of ascii is fine for our localized purpose of email, right?
    return s if isinstance(s, bytes) else s.encode(encoding)

def _to_native_string(s, encoding='ascii'):
    return s if isinstance(s, str) else s.decode(encoding)


class _InsecureAdlerCRC32Digest(object):
    """
    Just enough of a hashlib-like object to satisfy
    itsdangerous, producing a 32-bit integer checksum
    instead of a real 128 or 256 bit checksum.
    This is specifically NOT cryptographically secure,
    but for purposes of \"not looking stupid\" we've decided
    that email account security doesn't matter.
    """

    # These aren't documented and are reverse engineered

    digest_size = 4  # size of the output
    block_size = 64  # ???

    def __init__(self, init=b''):
        self.val = init

    def copy(self):
        return self.__class__(self.val)

    def update(self, val):
        self.val += val

    def digest(self):
        crc = zlib.adler32(self.val)

        # PY2 and PY3 have different return values for zlib.adler32. That function
        # returns a signed and unsigned integer respectively.
        #
        # https://docs.python.org/2.7/library/zlib.html#zlib.adler32
        # https://github.com/NextThought/nti.mailer/issues/4#issuecomment-550293781
        return struct.pack('!I', crc & 0xFFFFFFFF)


def _make_signer(default_key='$Id$',
                 salt='email recipient',
                 digest_method=_InsecureAdlerCRC32Digest):
    """
    Note that the default separator, '.' may appear in principal ids.
    """
    secret_key = _get_signer_secret(default_secret=default_key)
    signer = Signer(secret_key,
                    salt=salt,
                    digest_method=digest_method)
    return signer


def _get_default_sender():
    """
    Get the default sender from :class:`IMailerPolicy`.
    """
    policy = component.queryUtility(IMailerPolicy)
    return  policy is not None \
        and policy.get_default_sender()


def _brand_name(request):
    dng = component.queryMultiAdapter((getSite(), request),
                                      IDisplayNameGenerator)
    return dng() if dng != None else None


def _find_default_realname(request=None):
    """
    Called when the given fromaddr does not have a realname portion.
    We would prefer to use whatever is in the site policy, if there
    is one, otherwise we have a hardcoded default.
    """
    realname = None
    default_sender = _get_default_sender()
    if default_sender:
        realname, _ = parseaddr(default_sender)
        if realname is not None:
            realname = realname.strip()

    if not realname:
        if request is None:
            request = get_current_request()

        realname = _brand_name(request)

    return realname or "NextThought"


def __make_signer(default_key, **kwargs):
    if not default_key:
        return _make_signer(**kwargs)
    else:
        return _make_signer(default_key=default_key, **kwargs)


from six.moves import urllib_parse


def _sign(signer, principal_ids):
    """
    Given a signer, and a byte-string of principal ids, return
    a signed value, as lightly obfuscated as possible, to satisfy
    concerns about \"looking stupid\".

    Note that this value easily exposes the principal ID in readable fashion,
    giving someone in possession of the email both principal ID and registered
    email address. Watch out for phishing attacks.
    """

    sig = signer.get_signature(principal_ids)
    # The sig is always already base64 encoded, in the
    # URL/RFC822 safe fashion.
    principal_ids = urllib_parse.quote(principal_ids)

    return _to_bytes(principal_ids) + signer.sep + sig


def realname_from_recipients(fromaddr, recipients, request=None):
    realname, addr = parseaddr(fromaddr)
    if not realname and not addr:
        raise ValueError("Invalid fromaddr", fromaddr)
    if not realname:
        realname = _find_default_realname(request=request)
    return formataddr((realname, addr))


def verp_from_recipients(fromaddr,
                         recipients,
                         request=None,
                         default_key=None):

    realname = realname_from_recipients(fromaddr, recipients, request=request)
    realname, addr = parseaddr(realname)

    # We could special case the common case of recipients of length
    # one if it is a string: that typically means we're sending to the current
    # principal (though not necessarily so we'd have to check email match).
    # However, instead, I just want to change everything to send something
    # adaptable to IEmailAddressable instead.

    adaptable_to_email_addressable = [x for x in recipients
                                      if IEmailAddressable(x, None) is not None]
    principals = {IPrincipal(x, None) for x in adaptable_to_email_addressable}
    principals.discard(None)

    principal_ids = {x.id for x in principals}
    if len(principal_ids) == 1:
        # mildly encode them; this is just obfuscation.
        # Do that after signing to be sure we wind up with
        # something rfc822-safe
        # First, get bytes to avoid any default-encoding
        principal_id = _to_bytes(tuple(principal_ids)[0])
        # now sign
        signer = __make_signer(default_key)
        principal_id = _to_native_string(_sign(signer, principal_id))

        local, domain = addr.split('@')
        # Note: we may have a local address that already has a label '+'.
        # The principal ids with '+' should now be url quoted away. This
        # ensures we want the last '+' on parsing.
        addr = local + '+' + principal_id + '@' + domain

    result = formataddr((realname, addr))
    return result


def principal_ids_from_verp(fromaddr,
                            request=None,
                            default_key=None):
    if not fromaddr or '+' not in fromaddr:
        return ()

    _, addr = parseaddr(fromaddr)
    if '+' not in addr:
        return ()

    signer = __make_signer(default_key)

    # Split on our last '+' to allow user defined labels.
    signed_and_encoded = addr.rsplit('+', 1)[1].split('@')[0]

    signature_seperator = _to_native_string(signer.sep)
    if signature_seperator not in signed_and_encoded:
        return ()

    encoded_pids, sig = signed_and_encoded.rsplit(signature_seperator, 1)
    decoded_pids = urllib_parse.unquote(encoded_pids)

    signed = decoded_pids + signature_seperator + sig
    try:
        pids = signer.unsign(signed)
        pids = _to_native_string(pids)
    except BadSignature:
        return ()
    else:
        return pids.split(',')

interface.moduleProvides(IVERP)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Processors for :mod:`repoze.sendmail`, intended as a drop-in replacement
for the ``qp`` command line, using Amazon SES.

"""

from __future__ import print_function, absolute_import, division
__docformat__ = "restructuredtext en"

logger = __import__('logging').getLogger(__name__)

import gevent

import os

import time

from zope import interface

from email.message import Message

import boto3

from botocore.config import Config

from repoze.sendmail.encoding import encode_message

from repoze.sendmail.interfaces import IMailer

from repoze.sendmail.maildir import Maildir

from repoze.sendmail.queue import ConsoleApp as _ConsoleApp
from repoze.sendmail.queue import QueueProcessor

from zope.cachedescriptors.property import Lazy


@interface.implementer(IMailer)
class SESMailer(object):
    """
    This object does not handle throttling or quata actions;
    see also :mod:`nti.app.bulkemail.process`.
    """

    def __init__(self, region='us-east-1'):
        self.region=region

    @property
    def _ses_config(self):
        return Config(region_name=self.region)

    @Lazy
    def client(self):
        client = boto3.client('ses', config=self._ses_config)
        assert client
        return client

    def close(self):
        pass

    def send(self, fromaddr, toaddrs, message):
        if not isinstance(message, Message):  # pragma: no cover
            raise ValueError('Message must be instance of email.message.Message')

        message = encode_message(message)

        # Send the mail using SES, transforming SESError and known
        # subclasses into something the SMTP-based queue processor knows
        # how to deal with.
        # NOTE: now that we're here, we have the opportunity to de-VERP
        # the fromaddr found in the message, but still use the VERP form
        # in the fromaddr we pass to SES. In this way we can handle bounces
        # with the recipient none-the-wiser. See also :mod:`nti.app.bulkemail.process`
        # NOTE: Each recipient (To, CC, BCC) counts as a distinct message
        # for purposes of the quota limits. There are a maximum of
        # 50 dests per address. (http://docs.aws.amazon.com/ses/latest/APIReference/API_SendRawEmail.html)
        # NOTE: It is recommended to send an email to individuals:
        # http://docs.aws.amazon.com/ses/latest/DeveloperGuide/sending-email.html
        # "When you send an email to multiple recipients (recipients
        # are "To", "CC", and "BCC" addresses) and the call to Amazon
        # SES fails, the entire email is rejected and none of the
        # recipients will receive the intended email. We therefore
        # recommend that you send an email to one recipient at a time."

        # QQQ: The docs for SendRawEmail say that destinations is not required,
        # so how does that interact with what's in the message body?
        # Boto will accept either a string, a list of strings, or None
        self.client.send_raw_email(RawMessage={'Data': message},
                                   Source=fromaddr,
                                   Destinations=toaddrs)


import argparse
import sys
import logging

class ConsoleApp(_ConsoleApp):

    def __init__(self, argv=None):  # pylint: disable=unused-argument
        if argv is None:
            argv = sys.argv
        # Bypass the superclass, don't try to construct an SMTP mailer
        self.script_name = argv[0]
        self._process_args(argv[1:])
        self.mailer = SESMailer()
        getattr(self.mailer, 'client')


class MailerProcess(object):
    """
    A mailer processor that dumps the queue on a provided interval.
    """

    _exit = False

    def __init__(self, mailer_factory, queue_path, sleep_seconds=120):  # pylint: disable=unused-argument
        self.mailer_factory = mailer_factory
        self.sleep_seconds = sleep_seconds
        self.queue_path = queue_path
        self.mail_dir = Maildir(self.queue_path, create=True)

    def _maildir_factory(self, *args, **kwargs):
        return self.mail_dir

    def _do_process_queue(self):
        mailer = self.mailer_factory()
        assert mailer
        try:
            processor = QueueProcessor(mailer,
                                       self.queue_path, # Note this gets ignored by the Maildir factory we send
                                       Maildir=self._maildir_factory)
            logger.info('Processing messages %s' % (processor.maildir.path))
            processor.send_messages()
        finally:
            try:
                mailer.close()
            except AttributeError:
                pass
            mailer = None

    def run(self):
        while not self._exit:
            self._do_process_queue()
            logger.debug('Going to sleep for %i seconds' % (self.sleep_seconds))
            time.sleep(self.sleep_seconds)

def _stat_modified_time(attrs):
    """
    libev and libuv expose different attributes.
    Specifically modified time is st_mtime or st_mtim respectively.
    In libuv we need to look at st_mtim.tv_sec
    """
    try:
        return attrs.st_mtime
    except AttributeError:
        return attrs.st_mtim.tv_sec

def _stat_watcher_modified(watcher):
    """
    Inspects the stat watcher to see if the modified time
    has changed between the current attrs and the prev attrs
    """
    return _stat_modified_time(watcher.prev) != _stat_modified_time(watcher.attr)


_MINIMUM_DEBOUNCE_INTERVAL_SECONDS = 10


class MailerWatcher(MailerProcess):
    """
    A Mailer processor that watches for changes in the mail directory
    using gevent stat watchers.
    """
    watcher = None
    debouncer = None
    debouncer_count = 0

    max_process_frequency_seconds = _MINIMUM_DEBOUNCE_INTERVAL_SECONDS

    def __init__(self, *args, **kwargs):
        super(MailerWatcher, self).__init__(*args, **kwargs)
        hub = gevent.get_hub()
        to_watch = os.path.join(self.queue_path, str('new'))
        self.watcher = hub.loop.stat(to_watch)

    def _start_watching(self):
        assert self.watcher
        logger.debug('Starting watcher for MailDir %s', self.watcher.path)
        self.watcher.start(self._stat_change_observed)

    def _stop_watching(self):
        assert self.watcher
        logger.debug('Stopping watcher for MailDir %s', self.watcher.path)
        self.watcher.stop()

    def _do_process_queue(self):
        # The path we are watching has been modified
        #self._stop_watching()
        super(MailerWatcher, self)._do_process_queue()
        #self._start_watching()

    def _youve_got_mail(self):
        # We've detected we have mail. We want to debounce
        # this so we aren't going crazy. Process the queue at most ever
        # self.max_process_frequency_seconds
        # We use a gevent timer to accomplish this.

        hub = gevent.get_hub()
        if self.debouncer is None:
            self.debouncer = hub.loop.timer(self.max_process_frequency_seconds)

        def _timer_fired(self):
            self.debouncer.stop()
            self.debouncer = None
            if self.debouncer_count > 0:
                self.debouncer_count = 0
                self._youve_got_mail()

        if self.debouncer.active is False:
            self.debouncer_count = 0
            self.debouncer.start(_timer_fired, self)
            logger.info('Processing mail queue. Queue processing paused for %i seconds', self.max_process_frequency_seconds)
            self._do_process_queue()
        else:
            self.debouncer_count += 1
            logger.debug('Deferring queue processing because it was run recently')

    def _stat_change_observed(self):
        # On certain file systems we will see stat changes
        # for access times which we don't care about. We really
        # only care about modified times.`

        if _stat_watcher_modified(self.watcher):
            logger.debug('Maildir watcher detected MailDir modification')
            self._youve_got_mail()

    def run(self, seconds=None):
        # Process once initially in case we have things in the queue already
        self._do_process_queue()

        # Note we don't call start watching because _do_process_queue handles that
        self._start_watching()
        gevent.get_hub().join(seconds)

_LOG_LEVELS = [logging.ERROR, logging.WARN, logging.INFO, logging.DEBUG]

def _log_level_for_verbosity(verbosity=0):
    return _LOG_LEVELS[min(max(verbosity, 0), len(_LOG_LEVELS) - 1)]


def run_process(): # pragma: no cover

    parser = argparse.ArgumentParser(
        description='Spawn a process that stays alive, periodically polling the mail queue '
        'and sending new mail using SES.')
    parser.add_argument('queue_path',
                        help='The path to the maildir',
                        action='store')
    parser.add_argument('-r', '--sesregion',
                        help='The SES region to connect to.')
    parser.add_argument('-v', '--verbose',
                        help='How verbose to log.',
                        action='count',
                        default=1)
    parser.add_argument('-w', '--debounce-interval',
                        dest='interval',
                        help=('The number of seconds to wait between dumping the queue '
                              '(default: %(default)s)'),
                        action='store',
                        default=MailerWatcher.max_process_frequency_seconds,
                        type=int)

    arguments = parser.parse_args()

    log_level = _log_level_for_verbosity(arguments.verbose)
    logging.basicConfig(stream=sys.stderr,
                        format='%(asctime)s %(levelname)s %(message)s',
                        level=log_level)

    _mailer_factory = SESMailer if not arguments.sesregion else (lambda: SESMailer(arguments.sesregion))

    app = MailerWatcher(_mailer_factory, arguments.queue_path)

    if arguments.interval:
        app.max_process_frequency_seconds = max(_MINIMUM_DEBOUNCE_INTERVAL_SECONDS,
                                                arguments.interval)

    logger.info('Using debounce interval of %i', app.max_process_frequency_seconds)
    app.run()

def run_console(): # pragma: no cover
    if '--help' in sys.argv:
        # Override the help message we get from repoze.sendmail, we
        # ignore all the SMTP stuff it wants to do.
        parser = argparse.ArgumentParser(
            description='Process the mail queue one time and exit. '
            'Mail found in the mail queue will be sent using default SES settings.'
        )
        parser.add_argument('queue_path',
                        help='The path to the maildir',
                        action='store')
        parser.parse_args()
        return

    logging.basicConfig(stream=sys.stderr,
                        format='%(asctime)s %(levelname)s %(message)s',
                        level=logging.WARN)
    app = ConsoleApp()
    app.main()

if __name__ == "__main__":  # pragma NO COVERAGE
    run_console()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function, absolute_import, division

# disable: accessing protected members, too many methods
# pylint: disable=W0212,R0904
import fudge
from nti.mailer._compat import parseaddr

from hamcrest import is_
from hamcrest import none
from hamcrest import not_none
from hamcrest import assert_that
from hamcrest import has_property
from hamcrest import contains_string

from pyramid.testing import setUp as psetUp
from pyramid.testing import tearDown as ptearDown

from pyramid.interfaces import IRendererFactory

from pyramid_mailer.interfaces import IMailer

from pyramid_mailer.mailer import DummyMailer as _DummyMailer

from repoze.sendmail.interfaces import IMailDelivery

import unittest

from zope import component

from zope import interface

from zope.publisher.interfaces.browser import IBrowserRequest

from zope.security.interfaces import IPrincipal

from nti.app.pyramid_zope import z3c_zpt

from nti.mailer._default_template_mailer import _pyramid_message_to_message
from nti.mailer._default_template_mailer import create_simple_html_text_email

from nti.mailer.interfaces import IEmailAddressable
from nti.mailer.interfaces import EmailAddressablePrincipal
from nti.mailer.interfaces import IPrincipalEmailValidation


class ITestMailDelivery(IMailer, IMailDelivery):
    pass

class TestMailDelivery(_DummyMailer):

    default_sender = 'no-reply@nextthought.com'

    def __init__(self):
        super(TestMailDelivery, self).__init__()

    def send(self, fromaddr, toaddr, message):
        __traceback_info__ = fromaddr, toaddr
        self.queue.append(message)
        # compat with pyramid_mailer messages
        message.subject = message.get('Subject')
        payload = message.get_payload()
        message.body = payload[0].get_payload()
        if len(payload) > 1:
            message.html = payload[1].get_payload()


class PyramidMailerLayer(object):

    request = None

    @classmethod
    def setUp(cls):
        cls.config = psetUp(registry=component.getGlobalSiteManager(),
                            request=cls.request,
                            hook_zca=False)
        cls.config.setup_registry()
        cls.config.include('pyramid_chameleon')
        cls.config.include('pyramid_mako')
        component.provideUtility(z3c_zpt.renderer_factory,
                                 IRendererFactory,
                                 name=".pt")
        cls._mailer = mailer = TestMailDelivery()
        component.provideUtility(mailer, ITestMailDelivery)

    @classmethod
    def tearDown(cls):
        ptearDown()
        cls._mailer = None

    @classmethod
    def testSetUp(cls):
        pass

    @classmethod
    def testTearDown(cls):
        # Must implement
        pass

@interface.implementer(IPrincipalEmailValidation)
class TestEmailAddressablePrincipal(EmailAddressablePrincipal):

        def __init__(self, user, is_valid=True, *args, **kwargs):
                super(TestEmailAddressablePrincipal, self).__init__(user, *args, **kwargs)
                self.is_valid = is_valid

        def is_valid_email(self):
                return self.is_valid


@interface.implementer(IBrowserRequest)
class Request(object):
        context = None
        response = None
        application_url = 'foo'

        def __init__(self):
                self.annotations = {}

        def get(self, key, default=None):
                return default


class TestEmail(unittest.TestCase):

        layer = PyramidMailerLayer

        @fudge.patch('nti.mailer._verp._brand_name')
        def test_create_mail_message_with_non_ascii_name_and_string_bcc(self, brand_name):
                brand_name.is_callable().returns(None)

                class User(object):
                        username = 'the_user'

                class Profile(object):
                        # Note the umlaut e
                        realname = u'Suzë Schwartz'

                user = User()
                profile = Profile()
                request = Request()
                request.context = user

                token_url = 'url_to_verify_email'
                msg = create_simple_html_text_email('tests/templates/test_new_user_created',
                                                    subject='Hi there',
                                                    recipients=['jason.madden@nextthought.com'],
                                                    bcc='foo@bar.com',
                                                    template_args={'user': user,
                                                                   'profile': profile,
                                                                   'context': user,
                                                                   'href': token_url,
                                                                   'support_email': 'support_email' },
                                                    package='nti.mailer',
                                                    text_template_extension=".mak",
                                                    request=request)
                assert_that(msg, is_(not_none()))

                base_msg = _pyramid_message_to_message(msg, ['jason.madden@nextthought.com'], None)

                base_msg_string = str(base_msg)
                # quoted-prinatble encoding of iso-8859-1 value of umlaut-e
                assert_that(base_msg_string, contains_string('Hi=20Suz=EB=20Schwartz'))

                # Because we can't get to IPrincial, no VERP info
                name, email = parseaddr(msg.sender)
                assert_that(name, is_('NextThought'))
                assert_that(email, is_('no-reply@nextthought.com'))

                #
                assert_that(msg, has_property('bcc', ['foo@bar.com']))

        @fudge.patch('nti.mailer._verp._brand_name')
        def test_create_email_with_verp(self, brand_name):
                brand_name.is_callable().returns(None)

                @interface.implementer(IPrincipal, IEmailAddressable)
                class User(object):
                        username = 'the_user'
                        id = 'the_user'
                        email = 'thomas.stockdale@nextthought.com'  # this address encodes badly to simple base64

                class Profile(object):
                        realname = u'Suzë Schwartz'

                user = User()
                profile = Profile()
                request = Request()
                request.context = user

                token_url = 'url_to_verify_email'
                msg = create_simple_html_text_email('tests/templates/test_new_user_created',
                                                    subject='Hi there',
                                                    recipients=[TestEmailAddressablePrincipal(user, is_valid=True)],
                                                    template_args={'user': user,
                                                                   'profile': profile,
                                                                   'context': user,
                                                                   'href': token_url,
                                                                   'support_email': 'support_email' },
                                                    package='nti.mailer',
                                                    request=request)
                assert_that(msg, is_(not_none()))
                # import pyramid_mailer
                # from pyramid_mailer.interfaces import IMailer
                # from zope import component
                # mailer = pyramid_mailer.Mailer.from_settings( {'mail.queue_path': '/tmp/ds_maildir', 'mail.default_sender': 'no-reply@nextthought.com' } )
                # component.provideUtility( mailer, IMailer )
                # component.provideUtility(mailer.queue_delivery)
                # from .._default_template_mailer import _send_mail
                # _send_mail(msg, [user], None)
                # import transaction
                # transaction.commit()

                _pyramid_message_to_message(msg, [user], None)

                # we can get to IPrincipal, so we have VERP
                # The first part will be predictable, the rest won't
                name, email = parseaddr(msg.sender)
                assert_that(name, is_('NextThought'))
                assert_that(email, contains_string('no-reply+'))

                # Test invalid
                invalid_user = TestEmailAddressablePrincipal(user, is_valid=False)
                msg = create_simple_html_text_email('tests/templates/test_new_user_created',
                                                    subject='Hi there',
                                                    recipients=[invalid_user],
                                                    template_args={'user': user,
                                                                   'profile': profile,
                                                                   'context': user,
                                                                   'href': token_url,
                                                                   'support_email': 'support_email' },
                                                    package='nti.mailer',
                                                    request=request)
                assert_that(msg, none())

        @fudge.patch('nti.mailer._verp._brand_name')
        def test_create_email_with_mako(self, brand_name):
            brand_name.is_callable().returns(None)

            user = _User('the_user')
            request = Request()
            request.context = user

            msg = self._create_simple_email(request,
                                            text_template_extension=".mak")
            assert_that(msg, is_(not_none()))

        @fudge.patch('nti.mailer._verp._brand_name')
        def test_create_email_no_request_context(self, brand_name):
            brand_name.is_callable().returns(None)

            @interface.implementer(IBrowserRequest)
            class Request(object):
                response = None
                application_url = 'foo'

                def __init__(self):
                    self.annotations = {}

                def get(self, key, default=None):
                    return default

            request = Request()
            msg = self._create_simple_email(request,
                                            text_template_extension=".mak")
            assert_that(msg, is_(not_none()))

        def _create_simple_email(self,
                                 request,
                                 user=None,
                                 profile=None,
                                 text_template_extension=".txt"):

            user = user or _User('the_user')
            profile = profile or _Profile(u'Mickey Mouse')

            token_url = 'url_to_verify_email'
            msg = create_simple_html_text_email('tests/templates/test_new_user_created',
                                                subject='Hi there',
                                                recipients=['jason.madden@nextthought.com'],
                                                template_args={'user': user,
                                                               'profile': profile,
                                                               'context': user,
                                                               'href': token_url,
                                                               'support_email': 'support_email'},
                                                package='nti.mailer',
                                                text_template_extension=text_template_extension,
                                                request=request)
            return msg


class _User(object):
    def __init__(self, username):
        self.username = username


class _Profile(object):
    def __init__(self, realname):
        self.realname = realname



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function, absolute_import, division

import sys

try:
    from email.utils import parseaddr # PY3
    from email.utils import formataddr  # PY3
except ImportError:  # pragma: no cover
    from rfc822 import parseaddr # PY2
    from rfc822 import dump_address_pair as formataddr # PY2

PY2 = sys.version_info[0] <= 2
PY3 = sys.version_info[0] >= 3

if PY2:
    def is_nonstr_iter(v):
        return hasattr(v, '__iter__')
else:
    def is_nonstr_iter(v):
        if isinstance(v, str):
            return False
        return hasattr(v, '__iter__')



#!/Sr/bin/env python
# -*- coding: utf-8 -*-
"""
Mailing interfaces.

This package is based upon both :mod:`pyramid_mailer` and :mod:`repoze.sendmail`,
but the relevant parts are re-exported from this package.

.. $Id$
"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

from zope import interface

from zope.security.interfaces import IPrincipal

__all__ = (
    'IMailer',
    'IMailDelivery',
    'IPrincipalEmailValidation',
    'IEmailAddressable',
    'EmailAddressablePrincipal',
    'ITemplatedMailer',
    'IVERP',
    'IMailerPolicy',
    'IMailerTemplateArgsUtility',
)

# pylint:disable=inherit-non-class,no-self-argument,no-method-argument

try:
    from pyramid_mailer.interfaces import IMailer
except ImportError: # pragma: no cover
    class IMailer(interface.Interface):
        pass

try:
    from repoze.sendmail.interfaces import IMailDelivery
except ImportError: # pragma: no cover
    class IMailDelivery(interface.Interface):
        transaction_manager = interface.Attribute(u"The transaction manager to use.")

        def send(fromaddr, toaddrs, message):
            pass


from nti.schema.field import TextLine
# pylint:disable=I0011,E0213

logger = __import__('logging').getLogger(__name__)


class IPrincipalEmailValidation(interface.Interface):
    """
    A principal adapter that validates the given principal has a valid email.
    Currently, we only refuse to send an email if the given principal is
    adaptable to `IPrincipalEmailValidation` and this func returns False.

    This is useful to filter out bounced emails or users without an email.
    """

    def is_valid_email():
        """
        Returns a bool whether or not the given principal has a valid email.
        """


class IEmailAddressable(interface.Interface):
    """
    Something containing an email address. Commonly, when
    the object containing the email address represents a \"user\"
    of the system, it should also be adaptable to :class:`zope.security.interfaces.IPrincipal`.

    See :class:`.EmailAddressablePrincipal` for a simple serialization-safe
    example.
    """

    email = interface.Attribute(u"The email address to send to")


@interface.implementer(IEmailAddressable,
                       IPrincipal)
class EmailAddressablePrincipal(object):
    """
    An implementation of both :class:`IEmailAddressable`
    and :class:`IPrincipal` that combines both interfaces
    from something (a context) that is adaptable to both. Use this
    when the weight of the context is undesirable, such as during
    serialization (pickling), or when concurrency is a factor.

    This object copies the attributes from both interfaces into
    itself at construction time.
    """

    # These two attributes of IPrincipal aren't required
    title = None
    description = None

    def __init__(self, context):
        self.email = IEmailAddressable(context).email

        prin = IPrincipal(context)
        self.id = prin.id

        for name in 'title', 'description':
            prin_val = getattr(prin, name, None)
            if prin_val is not None:
                setattr(self, str(name), prin_val)

    def __str__(self):
        return str('Principal(%s/%s)' % (self.id, self.email))

    __repr__ = __str__

EmailAddresablePrincipal = EmailAddressablePrincipal


class ITemplatedMailer(interface.Interface):
    """
    An object, typically registered as a utility,
    that can handle putting together an email (having both
    text and HTML parts) by rendering templates.
    """

    def queue_simple_html_text_email(base_template,
                                     subject='',
                                     request=None,
                                     recipients=(),
                                     bcc=(),
                                     template_args=None,
                                     attachments=(),
                                     package=None,
                                     text_template_extension='.txt',
                                     message_factory=None):
        """
        Transactionally queues an email for sending. The email has both a
        plain text and an HTML version.

        :keyword recipients: A sequence of RFC822 email addresses as strings,
                or objects that can be adapted to an :class:`.IEmailAddressable`
                object. If no recipients are given, this does nothing. If any
                recipients are not strings, then if they cannot be adapted to
                :class:`.IEmailAddressable` or if the ``email`` attribute of the
                adapted object is false (none, empty) they will silently be dropped
                from the list. Objects that are :class:`.IEmailAddressable` should also
                (optionally) be able to become :class:`zope.security.interfaces.IPrincipal`
                by adaptation; this may permit us to construct a better (e.g., VERP)
                message.
        :keyword bcc: As for recipients.
        :keyword text_template_extension: The filename extension for the plain text template. Valid
                values are ".txt" for Chameleon templates (this is the
                default and preferred version) and ".mak" for Mako
                templates. Note that if you use Mako, the usual
                ``context`` argument is renamed to ``nti_context``, as
                ``context`` is a reserved word in Mako (this may change in the future).
        :keyword package: If given, and the template is not an absolute
                asset spec, then the template will be interpreted relative to this
                package (and its templates/ subdirectory if no subdirectory is specified).
                If no package is given, the package of the caller of this function is used.
        :keyword message_factory: A callable, called with the same arguments
                passed to this function (minus this param) to create a
                :class:`pyramid_mailer.message.Message` by rendering the pair
                of templates to create a text and html part. Defaults to
                :meth:`create_simple_html_text_email`.


        :return: The :class:`pyramid_mailer.message.Message` we sent, if we sent one,
                otherwise None.
        """

    def create_simple_html_text_email(base_template,
                                      subject='',
                                      request=None,
                                      recipients=(),
                                      template_args=None,
                                      attachments=(),
                                      package=None,
                                      bcc=(),
                                      text_template_extension='.txt'):
        """
        The same arguments and return types as :meth:`queue_simple_html_text_email`,
        but without the actual transactional delivery.
        """

    def do_html_text_templates_exist(base_template,
                                     text_template_extension='.txt',
                                     package=None):
        """
        A preflight method for checking if templates exist. Returns a True value
        if they do.
        """


class IVERP(interface.Interface):
    """
    Amazon SES now supports labels for `sending emails
    <http://docs.aws.amazon.com/ses/latest/DeveloperGuide/verify-email-addresses.html>`_,
    making it possible to do `VERP
    <https://en.wikipedia.org/wiki/Variable_envelope_return_path>`_,
    meaning we can directly identify the account we sent to (or even
    type of email) in case of a bounce. This requires using 'labels'
    and modifying the sending address: foo+label@domain.com. Note that
    SES makes no mention of the Sender header, instead putting the
    labels directly in the From line (which is what, for example,
    Facebook does) or in the Return-Path line (which is what trello
    does). However, SES only deals with the Return-Path header if you
    use its `API, not if you use SMTP
    <http://docs.aws.amazon.com/ses/latest/DeveloperGuide/notifications-via-email.html>`_
    """

    def realname_from_recipients(fromaddr, recipients, request=None):
        """
        This function takes a given From address and manipulates it to include
        a default realname, usually based on the current site and possibly
        the recipient list.

        Despite being located in the VERP module, this function does not actually
        do any signing.

        :return: The ``fromaddr``, guaranteed to be in \"Realname <to@example.com>\"
                format.
        """

    def verp_from_recipients(fromaddr, recipients, request=None):
        """
        This function takes a given from address and manipulates it to
        include VERP information that identifies the *accounts* of the
        recipients. For this to work, the recipients must initially be
        passed as things that can be adapted to `IEmailAddressable`
        objects; for those objects that can be adapted, then if they can
        be adapted to :class:`.IPrincipal`, we include the principal ID.

        In addition, if the `fromaddr` does not include a realname,
        adds a default.

        .. note:: We take the request as an argument because at some point
                we may want to include some notion of the sending site,
                although it's probably better to use separate SES/SNS queues
                if possible.

        :param fromaddr: The initial from address, without any labels,
                possibly including a realname portion.

        :return: The ``fromaddr``, manipulated to include a VERP
                label, if possible, identifying the principals related to the
                recipients and/or request.
        """

    def principal_ids_from_verp(fromaddr, request=None):
        """
        Decode the VERP information as encoded in :meth:`verp_from_recipients`,
        returning a sequence of positively-identified principal IDs for the current
        environment, if any.
        """


class IMailerPolicy(interface.Interface):
    """
    Mailer policy utility
    """

    # Deprecated
    DEFAULT_EMAIL_SENDER = TextLine(title=u'An optional email sender',
                                    description=u'An email address used to send emails to users'
                                                u'such as account creation, both on behalf of this'
                                                u'object as well as from other places. Optional.',
                                    required=False,
                                    default=None)

    def get_default_sender():
        """
        Returns a default sender to be used when no fromaddr
        is provided.
        """

    def get_signer_secret():
        """
        Returns a signer secret, used for verp.
        """


class IMailerTemplateArgsUtility(interface.Interface):
    """
    A utility that can supplement mail template args.
    """

    def get_template_args():
        """
        Returns a (possibly empty) dict of supplemental template args.
        """



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function, absolute_import, division
__docformat__ = "restructuredtext en"

# disable: accessing protected members, too many methods
# pylint: disable=W0212,R0904
import os

from tempfile import mkdtemp

import unittest

import email

from hamcrest import assert_that
from hamcrest import is_
from hamcrest import has_length

import fudge




from repoze.sendmail.delivery import QueuedMailDelivery
from repoze.sendmail.maildir import Maildir

from repoze.sendmail.tests.test_delivery import _makeMailerStub

from nti.mailer.queue import SESMailer
from nti.mailer.queue import MailerWatcher

# pylint:disable=line-too-long
MSG_STRING = 'MIME-Version: 1.0\nFrom: NextThought <no-reply+70108544275840.qhjWPQ@nextthought.com>\nSubject: Welcome to NextThought\nTo: test.user@nextthought.com\nMessage-Id: <20140528152113.23368.67989.repoze.sendmail@nextthought.com>\nDate: Wed, 28 May 2014 15:21:13 -0000\nContent-Type: multipart/alternative;\n boundary="===============3015559400140931547=="\n\n--===============3015559400140931547==\nContent-Type: text/plain; charset="us-ascii"\nMIME-Version: 1.0\nContent-Transfer-Encoding: quoted-printable\nContent-Disposition: inline\n\nHi=20Test=20User!\n\nThank=20you=20for=20creating=20your=20new=20account=20and=20welcome=20to=20=\nNextThought!\n\nUsername:=2070108544275840\nLog=20in=20at:=20https://alpha.nextthought.com\n\nNextThought=20offers=20interactive=20content=20and=20rich=20features=20to=\n=20make\nlearning=20both=20social=20and=20personal.=20Explore=20the=20NextThought=20=\nHelp=20Center\nin=20your=20Library=20to=20get=20started=20and=20learn=20more=20about=20the=\n=20exciting\ninteractive=20features=20NextThought=20has=20to=20offer.\n\nSincerely,\nNextThought\n\nIf=20you=20feel=20this=20email=20was=20sent=20in=20error,=20or=20this=20acc=\nount=20was=20created\nwithout=20your=20consent,=20you=20may=20email=20us=20at=20support@nextthoug=\nht.com.\n\n--===============3015559400140931547==\nContent-Type: text/html; charset="us-ascii"\nMIME-Version: 1.0\nContent-Transfer-Encoding: quoted-printable\nContent-Disposition: inline\n\n<!DOCTYPE=20html=20PUBLIC=20"-//W3C//DTD=20XHTML=201.0=20Strict//EN"\n=09=20"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">\n<html=20xmlns=3D"http://www.w3.org/1999/xhtml">\n=09<head>\n=09=09<meta=20http-equiv=3D"content-type"=20content=3D"text/html;=20charset=\n=3Dutf-8"=20/>\n=09=09<title>Welcome=20To=20NextThought</title>\n=09=09<style>\n=09=09=09#green-bar=20{\n=09=09=09background-color:=20#89be3c;\n=09=09=09width:=20100%;\n=09=09=09height:=205px;\n=09=09=09margin-left:=20-30px;\n=09=09=09padding-right:=2040px;\n=09=09=09}\n=09=09=09#logo-bar=20{\n=09=09=09margin-top:=2020px;\n=09=09=09margin-bottom:=2030px;\n=09=09=09}\n=09=09=09body=20{\n=09=09=09margin-left:=2030px;\n=09=09=09margin-right:=200px;\n=09=09=09font-family:=20Helvetica,=20Arial,=20sans-serif;\n=09=09=09font-size:=2014pt;\n=09=09=09line-height:=2020pt;\n=09=09=09color:=20#757474;\n=09=09=09}\n=09=09=09.normal-text=20{\n=09=09=09margin-left:=2030px;\n=09=09=09margin-right:=200px;\n=09=09=09font-family:=20Helvetica,=20Arial,=20sans-serif;\n=09=09=09font-size:=2014pt;\n=09=09=09line-height:=2020pt;\n=09=09=09color:=20#757474;\n=09=09=09}\n=09=09=09a=20{\n=09=09=09text-decoration:=20none;\n=09=09=09color:=20#3fb3f6;\n=09=09=09}\n=09=09=09.tterm,=20strong=20{\n=09=09=09font-weight:=20bold;\n=09=09=09color:=20#494949;\n=09=09=09}\n=09=09=09.tterm-color=20{\n=09=09=09font-weight:=20bold;\n=09=09=09color:=20#3fb3f6;\n=09=09=09}\n=09=09=09h1,=20h2,=20h3,=20h4,=20h5,=20h6=20{\n=09=09=09font-weight:=20bold;\n=09=09=09font-family:=20Helvetica,=20Arial,=20sans-serif;\n=09=09=09color:=20#757474;\n=09=09=09margin-left:=2030px;\n=09=09=09}\n=09=09=09h1=20{\n=09=09=09font-size:=2016pt;\n=09=09=09}\n=09=09=09h2=20{\n=09=09=09font-size:=2014pt;\n=09=09=09}\n=09=09=09body=20p=20{\n=09=09=09margin-right:=2030px;\n=09=09=09margin-left:=2030px;\n=09=09=09font-family:=20Helvetica,=20Arial,=20sans-serif;\n=09=09=09font-size:=2014pt;\n=09=09=09line-height:=2020pt;\n=09=09=09color:=20#757474;\n=09=09=09}\n=09=09=09.pnormal-text=20{\n=09=09=09margin-left:=2030px;\n=09=09=09margin-right:=2030px;\n=09=09=09font-family:=20Helvetica,=20Arial,=20sans-serif;\n=09=09=09font-size:=2014pt;\n=09=09=09line-height:=2020pt;\n=09=09=09color:=20#757474;\n=09=09=09}\n=09=09</style>\n=09</head>\n=09<body>\n=09=09<div>\n=09=09=09<div=20id=3D"green-bar"=20style=3D"background-color:=20#89be3c;=20=\nwidth:=20100%;=20height:=205px;=20margin-left:=20-30px;=20padding-right:=20=\n40px;"></div>\n=09=09=09<div=20id=3D"logo-bar">\n=09=09=09=09<img=20src=3D"https://d2ixlfeu83tci.cloudfront.net/images/email=\n_logo.png"=20width=3D"177"=20height=3D"25"=20alt=3D"NextThought=20Logo"=20/>\n=09=09=09</div>\n=09=09=09\n=09=09=09\n=09=09=09\n=09=09=09\n=09=09</div>\n=09=09<p>Hi=20<span=20class=3D"realname=20tterm">Test=20User</span>!</p>\n\n=09=09<p>Thank=20you=20for=20creating=20your=20new=20account=20and=20welcom=\ne=20to=20NextThought!</p>\n\n=09=09<p>\n=09=09=09<strong>Username:</strong>=20<span=20class=3D"tterm-color">7010854=\n4275840</span>=20<br=20/>\n=09=09=09<strong>Log=20in=20at:</strong>=20<a=20href=3D"https://alpha.nextt=\nhought.com">https://alpha.nextthought.com</a>\n=09=09</p>\n=09=09<p>NextThought=20offers=20interactive=20content=20and=20rich=20featur=\nes=20to=20make=20learning=20both=20social=20and=20personal.=20Explore=20the=\n=20NextThought=20Help=20Center=20in=20your=20Library=20to=20get=20started=\n=20and=20learn=20more=20about=20the=20exciting=20interactive=20features=20N=\nextThought=20has=20to=20offer.</p>\n\n=09=09<p>\n=09=09=09<span>Sincerely,</span><br=20/>\n=09=09=09NextThought\n=09=09</p>\n\n=09=09<p=20style=3D"font-size:=20smaller">\n=09=09=09<span>If=20you=20feel=20this=20email=20was=20sent=20in=20error,=20=\nor=20this=20account=20was=20created=20without=20your=20consent,=20you=20may=\n=20email=20us=20at=20<a=20href=3D"mailto:mailto:support@nextthought.com">su=\npport@nextthought.com</a></span>\n=09=09</p>\n\n=09</body>\n</html>\n\n--===============3015559400140931547==--\n'

MSG_BYTES = MSG_STRING if isinstance(MSG_STRING, bytes) else MSG_STRING.encode('ascii')

class TestMailer(unittest.TestCase):

    def setUp(self):
        super(TestMailer, self).setUp()
        self.message = email.message_from_string(MSG_STRING)

    def test_region(self):
        from botocore.exceptions import EndpointConnectionError
        from botocore.exceptions import NoCredentialsError
        # Defaults to us-east-1
        mailer = SESMailer()

        assert_that(mailer.client.meta.endpoint_url, is_('https://email.us-east-1.amazonaws.com'))

        mailer = SESMailer('us-west-2')
        assert_that(mailer.client.meta.endpoint_url, is_('https://email.us-west-2.amazonaws.com'))

        mailer = SESMailer('bad-region')
        with self.assertRaises((EndpointConnectionError, NoCredentialsError)):
            mailer.client.get_send_quota()

    def test_send(self):
        mailer = SESMailer()

        send_kwargs = {}

        def send_raw_email(*_args, **kwargs):
            send_kwargs.update(kwargs)

        mailer.client = fudge.Fake('SESClient')
        mailer.client.provides('send_raw_email').calls(send_raw_email)

        mailer.send('from', ('to',), self.message)

        encoded_msg_sent = send_kwargs['RawMessage']['Data']
        if not isinstance(encoded_msg_sent, str):
            sent_msg_str = encoded_msg_sent.decode('ascii')
        else:
            sent_msg_str = encoded_msg_sent
        # In order to get reasonable error messages, we want to do a line-wise
        # comparison. Moreover, Python 3 and Python 2 produce slightly different
        # line breaks and spacing. This is the case in headers: Python 3
        # likes to add extra line breaks if a header value is too long (so we
        # shortened the Message-Id value to avoid that); Python 2 did that for the
        # Content-Type header, but for whatever reason Python 3 re-combines that
        # line.
        self.maxDiff = None
        def prep_lines(msg_str):
            msg_str = msg_str.replace('\n boundary=', ' boundary=')
            result = [
                x.strip()
                for x in msg_str.splitlines()
            ]
            return result
        self.assertEqual(prep_lines(sent_msg_str), prep_lines(MSG_STRING))


class TestMailerWatcher(unittest.TestCase):

    def setUp(self):
        self.dir = mkdtemp()
        self.queue_dir = os.path.join(self.dir, str("queue"))
        self.delivery = QueuedMailDelivery(self.queue_dir)
        self.maildir = Maildir(self.queue_dir, True)
        self.mailer = _makeMailerStub()

    def test_delivery(self):
        from email.message import Message
        from_addr = "foo@bar.foo"
        to_addr = "bar@foo.bar"
        message = Message()
        message['Subject'] = 'Pants'
        message.set_payload('Nice pants, mister!')

        import transaction
        transaction.manager.begin()
        self.delivery.send(from_addr, to_addr, message)
        self.delivery.send(from_addr, to_addr, message)
        transaction.manager.commit()

        assert_that(tuple(self.maildir), has_length(2))

        def _mailer_factory():
            return self.mailer

        watcher = MailerWatcher(_mailer_factory, self.queue_dir)
        watcher.run(seconds=.1)

        assert_that(tuple(self.maildir), has_length(0))



import codecs
from setuptools import setup, find_packages

entry_points = {
    'console_scripts': [
        'nti_mailer_qp_console = nti.mailer.queue:run_console',
        'nti_mailer_qp_process = nti.mailer.queue:run_process',
        'nti_qp = nti.mailer.queue:run_console' # backwards compatibility
    ],
}

TESTS_REQUIRE = [
    'fudge',
    'nti.testing',
    'zope.testrunner',
    'nti.app.pyramid-zope >= 0.0.3',
    'pyramid_chameleon',
    'pyramid_mako',
]


def _read(fname):
    with codecs.open(fname, encoding='utf-8') as f:
        return f.read()


setup(
    name='nti.mailer',
    version="0.0.1a1",
    author='Josh Zuech',
    author_email='josh.zuech@nextthought.com',
    description="NTI mailer",
    long_description=(
        _read('README.rst')
        + '\n\n'
        + _read("CHANGES.rst")
    ),
    license='Apache',
    keywords='Base',
    classifiers=[
        'Framework :: Zope3',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],
    url="https://github.com/NextThought/nti.mailer",
    zip_safe=True,
    packages=find_packages('src'),
    package_dir={'': 'src'},
    include_package_data=True,
    namespace_packages=['nti'],
    tests_require=TESTS_REQUIRE,
    install_requires=[
        'gevent',
        'setuptools',
        'boto3',
        'BTrees',
        'itsdangerous',
        'nti.schema',
        'repoze.sendmail',
        # premailer dropped Python 2 support in version 3.7;
        # unfortunately, they don't have the proper ``python_requires`` metadata
        # to let installers know this.
        'premailer < 3.7.0; python_version == "2.7"',
        'premailer >= 3.7.0; python_version != "2.7"',
        # The < 2.0 part is from nti.app.pyramid_zope, a test
        # dependency. But older released versions on PyPI (< 0.0.3)
        # do not specify this correctly.
        'pyramid < 2.0',
        'pyramid_mailer',
        'six',
        'ZODB',
        'zc.displayname',
        'zope.annotation',
        'zope.catalog',
        'zope.component',
        'zope.container',
        'zope.dottedname',
        'zope.interface',
        'zope.intid',
        'zope.location',
        'zope.schema',
        'zope.security',
    ],
    extras_require={
        'test': TESTS_REQUIRE,
        'docs': [
            'Sphinx',
            'repoze.sphinx.autointerface',
            'sphinxcontrib-programoutput',
            'sphinx_rtd_theme',
        ],
    },
    entry_points=entry_points,
    python_requires=">=2.7,!=3.0.*,!=3.1.*,!=3.2.*,!=3.3.*,!=3.4.*,!=3.5",
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Utility functions having to do with sending emails.

This module provides the :class:`nti.mailer.interfaces.ITemplatedMailer` interface.
"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

from premailer import transform

from pyramid.path import caller_package

from pyramid.renderers import render
from pyramid.renderers import get_renderer

from pyramid.threadlocal import get_current_request

from pyramid_mailer.message import Message

from six import string_types

from zope import component
from zope import interface

from zope.dottedname import resolve as dottedname

from nti.mailer.interfaces import IVERP
from nti.mailer.interfaces import IMailer
from nti.mailer.interfaces import IMailDelivery
from nti.mailer.interfaces import IMailerPolicy
from nti.mailer.interfaces import ITemplatedMailer
from nti.mailer.interfaces import IEmailAddressable
from nti.mailer.interfaces import IPrincipalEmailValidation
from nti.mailer.interfaces import IMailerTemplateArgsUtility

from nti.mailer._compat import is_nonstr_iter

logger = __import__('logging').getLogger(__name__)


def _get_renderer_spec_and_package(base_template,
                                   extension,
                                   package=None,
                                   level=3):
    if isinstance(package, string_types):
        package = dottedname.resolve(package)

    # Did they give us a package, either in the name or as an argument?
    # If not, we need to get the right package
    if ':' not in base_template and package is None:
        # 2 would be our caller, aka this module.
        package = caller_package(level)
    # Do we need to look in a subdirectory?
    if ':' not in base_template and '/' not in base_template:
        base_template = 'templates/' + base_template

    return base_template + extension, package


def _get_renderer(base_template,
                  extension,
                  package=None,
                  level=3):
    """
    Given a template name, find a renderer for it.
    For template name, we accept either a relative or absolute
    asset spec. If the spec is relative, it can be 'naked', in which
    case it is assummed to be in the templates sub directory.

    This *must* only be called from this module due to assumptions
    about the call tree.
    """

    template, package = _get_renderer_spec_and_package(base_template,
                                                       extension,
                                                       package=package,
                                                       level=level + 1)

    return get_renderer(template, package=package)


def do_html_text_templates_exist(base_template,
                                 text_template_extension='.txt',
                                 package=None,
                                 _level=3):
    """
    A preflight method for checking if templates exist. Returns a True value
    if they do.
    """
    try:
        _get_renderer(base_template, '.pt', package=package, level=_level)
        _get_renderer(base_template, text_template_extension,
                      package=package, level=_level)
    except ValueError:
        # Pyramid raises this if the template doesn't exist
        return False
    return True


def _as_recipient_list(recipients):
    # XXX: Perhaps we should enforce a certain kwarg to ensure we always get
    # users here. We definitely prefer it since we can enforce we do not send
    # to bounced addresses. In some cases, we only have raw email addresses.
    # Currently, we'll just ignore those users with invalid addresses.
    result = []
    if recipients:
        # accept a raw string
        recipients = recipients if is_nonstr_iter(recipients) else [recipients]
        for recipient in recipients:
            # If we have a principal object, explicitly check if `is_valid_email`.
            email_validation = IPrincipalEmailValidation(recipient, None)
            if      email_validation is not None \
                and not email_validation.is_valid_email():
                continue
            # Convert any IEmailAddressable into their email, and strip
            # empty strings
            recipient = getattr(IEmailAddressable(recipient, recipient), 'email', recipient)
            if isinstance(recipient, string_types) and recipient:
                result.append(recipient)
    return result

as_recipient_list = _as_recipient_list

def create_simple_html_text_email(base_template,
                                  subject='',
                                  request=None,
                                  recipients=(),
                                  template_args=None,
                                  reply_to=None,
                                  attachments=(),
                                  package=None,
                                  cc=(),
                                  bcc=(),
                                  text_template_extension='.txt',
                                  _level=3):
    """
    Create a :class:`pyramid_mailer.message.Message` by rendering
    the pair of templates to create a text and html part.

    :keyword text_template_extension: The filename extension for the plain text template. Valid values
            are ".txt" for Chameleon templates (this is the default and preferred version) and ".mak" for
            Mako templates. Note that if you use Mako, the usual ``context`` argument is renamed to ``nti_context``,
            as ``context`` is a reserved word in Mako.
    :keyword package: If given, and the template is not an absolute
            asset spec, then the template will be interpreted relative to this
            package (and its templates/ subdirectory if no subdirectory is specified).
            If no package is given, the package of the caller of this function is used.
    """

    recipients = _as_recipient_list(recipients)

    if not recipients:
        logger.info("Refusing to attempt to send email with no recipients")
        return
    if not subject:
        # Should the subject already be localized or should we do that?
        logger.info("Refusing to attempt to send email with no subject")
        return

    if request is None:
        request = get_current_request()

    cc = _as_recipient_list(cc)
    bcc = _as_recipient_list(bcc)

    def make_args(extension):
        # Mako gets bitchy if 'context' comes in as an argument, but
        # that's what Chameleon wants. To simplify things, we handle that
        # for our callers. They just want to use 'context'.
        # This should be fixed with 1.0a2
        the_context_name = 'nti_context' if extension == text_template_extension and text_template_extension != '.txt' else 'context'
        result = {}
        if request:
            result[the_context_name] = getattr(request, 'context', None)
        if template_args:
            result.update(template_args)

        if the_context_name == 'nti_context' and 'context' in template_args:
            result[the_context_name] = template_args['context']
            del result['context']
        for args_utility in component.getAllUtilitiesRegisteredFor(IMailerTemplateArgsUtility):
            result.update(args_utility.get_template_args(request))
        return result

    def do_render(pkg):
        specs_and_packages = [_get_renderer_spec_and_package(base_template,
                                                             extension,
                                                             package=pkg,
                                                             level=_level + 1) + (extension,)
                              for extension in ('.pt', text_template_extension)]

        return [render(spec,
                       make_args(extension),
                       request=request,
                       package=pkg)
                for spec, pkg, extension in specs_and_packages]

    try:
        html_body, text_body = do_render(package)
    except ValueError as e:
        # This is just to handle the case where the
        # site specifies a package, but wants to use
        # a default template in some cases.
        if package is None:
            raise e
        # Ok, let's try to find the package.
        html_body, text_body = do_render(None)

    # Email clients do not handle CSS well unless it's inlined.
    # This can be expensive (~.4s per email) if users interactively
    # trigger large numbers of emails. In that case, the email is
    # probably better off created with inlined styles.
    html_body = transform(html_body)

    # PageTemplates (Chameleon and Z3c.pt) produce Unicode strings.
    # Under python2, at least, the text templates (Chameleon alone) produces byte objects,
    # (JAM: TODO: Can we make it stay in the unicode realm? Pyramid config?)
    # (JAM: TODO: Not sure about what Mako does?)
    # apparently encoded as UTF-8, which is not ideal. This either is
    # a bug itself (we shouldn't pass non-ascii values as text/plain)
    # or triggers a bug in pyramid mailer when it tries to figure out the encoding,
    # leading to a UnicodeError.
    # The fix is to supply the charset parameter we want to encode as;
    # Or we could decode it ourself, which lets us use the optimal encoding
    # pyramid_mailer picks...we ignore errors
    # here to make sure that we can send /something/
    if isinstance(text_body, bytes):
        text_body = text_body.decode('utf-8', 'replace')

    # JAM: Why are we quoted-printable encoding? That produces much bigger
    # output...whether we do it like this, or simply pass in the unicode
    # strings, we get quoted-printable. We would pass Attachments if we
    # wanted to specify the charset (see above)
    # message = Message( subject=subject,
    # 				   recipients=recipients,
    # 				   body=Attachment(data=text_body, disposition='inline',
    # 								   content_type='text/plain',
    # 								   transfer_encoding='quoted-printable'),
    # 				   html=Attachment(data=html_body, disposition='inline',
    # 								   content_type='text/html',
    # 								   transfer_encoding='quoted-printable') )
    message = Message(subject=subject,
                      recipients=recipients,
                      body=text_body,
                      html=html_body,
                      cc=cc,
                      bcc=bcc,
                      attachments=attachments)

    if reply_to:
        message.extra_headers['Reply-To'] = reply_to

    return message


def queue_simple_html_text_email(*args, **kwargs):
    """
    Transactionally queues an email for sending. The email has both a
    plain text and an HTML version.

    :keyword text_template_extension: The filename extension for the plain text template. Valid values
            are ".txt" for Chameleon templates (this is the default and preferred version) and ".mak" for
            Mako templates. Note that if you use Mako, the usual ``context`` argument is renamed to ``nti_context``,
            as ``context`` is a reserved word in Mako.

    :return: The :class:`pyramid_mailer.message.Message` we sent.
    """

    kwargs = dict(kwargs)
    if '_level' not in kwargs:
        kwargs['_level'] = 4
    message_factory = kwargs.pop('message_factory', create_simple_html_text_email)
    message = message_factory(*args, **kwargs)
    # There are cases where this will be none (bounced email handling, missing
    # subject - error?). In at least the bounced email case, we want to avoid
    # sending the email and erroring.
    if message is None:
        return
    return _send_pyramid_mailer_mail(message,
                                     recipients=kwargs.get('recipients'),
                                     request=kwargs.get('request'))


def _send_pyramid_mailer_mail(message, recipients=None, request=None):
    """
    Given a :class:`pyramid_mailer.message.Message`, transactionally deliver
    it to the queue.

    :return: The :class:`pyramid_mailer.message.Message` we sent.
    """
    # The pyramid_mailer.Message class is slightly nicer than the
    # email package messages, if much less powerful. However, it makes the
    # mistake of using different methods for send vs send_to_queue.
    # It is built of top of repoze.sendmail and an IMailer contains two instances
    # of repoze.sendmail.interfaces.IMailDelivery, one for queue and one
    # for immediate, and those objects do the real work and also have a consistent
    # interfaces. It's easy to change the pyramid_mail message into a email
    # message
    _send_mail(pyramid_mail_message=message,
               recipients=recipients, request=request)
    return message


def _compute_from(*args, **kwargs):
    verp = component.queryUtility(IVERP)
    if verp is None:
        from . import _verp
        verp = _verp
    return verp.verp_from_recipients(*args, **kwargs)


def _get_from_address(pyramid_mail_message, recipients, request):
    """
    Get a valid `From`/`Sender`/`Return-Path` address. This field is required and
    must be from a verified email address (e.g. @nextthought.com).
    """
    pyramidmailer = component.queryUtility(IMailer)
    if request is None:
        request = get_current_request()

    fromaddr = getattr(pyramid_mail_message, 'sender', None)

    if not fromaddr:
        # Can we get a site policy for the current site?
        # It would be the unnamed IComponents
        policy = component.queryUtility(IMailerPolicy)
        if policy:
            fromaddr = policy.get_default_sender()
    if not fromaddr:
        fromaddr = getattr(pyramidmailer, 'default_sender', None)

    if not fromaddr:
        raise RuntimeError("No one to send mail from")

    result = _compute_from(fromaddr, recipients, request)
    return result


def _pyramid_message_to_message(pyramid_mail_message, recipients, request):
    """
    Preps a pyramid message for sending, including adjusting its sender if needed.

    :return:
    """
    assert pyramid_mail_message is not None

    fromaddr = _get_from_address(pyramid_mail_message, recipients, request)

    pyramid_mail_message.sender = fromaddr
    # Sadly, as of 2014-05-22, Amazon SES (and some other SMTP relays, actually, if I understand
    # correctly) don't support setting Sender or Return-Path. They get ignored.
    # (At least for SES, this is because it need to set the Return-Path value
    # to something it controls in order to handle stateful retry logic, and delivery
    # to correct bounce queue, etc:
    #	  Return-Path: <000001462444a009-cfdcd8ed-008e-4bee-9ea7-30a47b615e64-000000@amazonses.com>
    # )
    # If this did work, we could leave the From address alone.
    # pyramid_mail_message.extra_headers['Sender'] = fromaddr
    # pyramid_mail_message.extra_headers['Return-Path'] = fromaddr
    message = pyramid_mail_message.to_message()
    return message


def _send_mail(pyramid_mail_message=None, recipients=(), request=None):
    """
    Sends a message transactionally.
    """
    assert pyramid_mail_message is not None
    pyramidmailer = component.queryUtility(IMailer)

    message = _pyramid_message_to_message(
        pyramid_mail_message, recipients, request
    )

    delivery = component.queryUtility(IMailDelivery) \
            or getattr(pyramidmailer, 'queue_delivery', None)
    if delivery:
        delivery.send(pyramid_mail_message.sender,
                      pyramid_mail_message.send_to,
                      message)
    elif pyramidmailer and pyramid_mail_message:
        pyramidmailer.send_to_queue(pyramid_mail_message)
    else:
        raise RuntimeError("No way to deliver message")


interface.moduleProvides(ITemplatedMailer)
